using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace MandelbrotKliens
{
  public partial class frmMandelbrotKliens : Form
  { /// <summary>
    /// A vizsglt vals tartomny als hatra.
    /// </summary>
    private double min_re;
    /// <summary>
    /// A vizsglt kpzetes tartomny als hatra.
    /// </summary>
    private double min_im;
    /// <summary>
    /// A vizsglt vals tartomny fels hatra.
    /// </summary>
    private double max_re;
    /// <summary>
    /// A vizsglt kpzetes tartomny fels hatra.
    /// </summary>
    private double max_im;
    /// <summary>
    /// Az itercik maximlisan megengedett szma.
    /// </summary>
    private int max_iter;
    /// <summary>
    /// A rajzterlet szlessge.
    /// </summary>
    private int kp_szl;
    /// <summary>
    /// A rajzterlet magassga.
    /// </summary>
    private int kp_mag;
    /// <summary>
    /// Az egyes kppontokhoz tartoz sznek RGB rendszerben egy egydimenzis 
    /// tmbben.
    /// </summary>
    private int[] iSznek;
    /// <summary>
    /// A kpet ler bitkp objektum.
    /// </summary>
    private Bitmap bmKp;
    /// <summary>
    /// A szervert elrhetv tev proxy osztly objektuma.
    /// </summary>
    private localhost.MandelbrotSzerver Szerver;

    public frmMandelbrotKliens()
    { InitializeComponent();
      // A vizsglt tglalap alak terlet bal fels sarka.
      min_re = min_im = 0;
      // A vizsglt tglalap alak terlet jobb als sarka.
      max_re = max_im = 1.5;
      // Az itercik maximlisan megengedett szma.
      max_iter = 10;
      // Eltroljuk a PictureBox komponens kezdeti szlessgt s 
      // magassgt.
      kp_szl = pbKp.Width;
      kp_mag = pbKp.Height;
      // Ltrehozunk egy bitkp objektumot.
      bmKp = new Bitmap(kp_szl, kp_mag);
      // Ltrehozzuk a szervert jelkpez objektumot.
      Szerver = new MandelbrotKliens.localhost.MandelbrotSzerver();

    }
    /// <summary>
		/// Megjelenti modlisan a Nvjegy prbeszdablakot.
		/// </summary>
    private void tsmiNvjegy_Click(object sender, EventArgs e)
    {	frmNvjegy nf=new frmNvjegy();
			nf.ShowDialog();
    }
    /// <summary>
    /// Kilps a programbl.
    /// </summary>
    private void tsmiKilps_Click(object sender, EventArgs e)
    { Application.Exit();
    }
    
    /// <summary>
    /// Mveletek men Bellts menpont.
    /// </summary>
    private void tsmiBelltsok_Click(object sender, EventArgs e)
    { // Ltrehozunk egy pldnyt a prbeszdablakbl.
      frmBelltsok bf=new frmBelltsok();
      // Belltjuk a tulajdonsgokat a trolt adatok alapjn.
      bf.dMinRe=min_re;
      bf.dMaxRe=max_re;
      bf.dMinIm=min_im;
      bf.dMaxIm=max_im;
      bf.iIterMax=max_iter;
      bf.sURL=Szerver.Url;
      // Megjelentjk a prbeszdablakot
      if(bf.ShowDialog()==DialogResult.OK)
      { // Ha a felhasznl az OK gombbal zrta be a prbeszdablakot,
        // akkor kiolvassuk a megadott rtkeket.
        min_re=bf.dMinRe;
        max_re=bf.dMaxRe;
        min_im=bf.dMinIm;
        max_im=bf.dMaxIm;
        max_iter=bf.iIterMax;
        Szerver.Url=bf.sURL;
      }
    }

    /// <summary>
    /// Mveletek men Indt menpont.
    /// </summary>
    private void tsmiIndt_Click(object sender, EventArgs e)
    { // Belltjuk a homokra kurzort, jelezve, hogy dolgozik a program
      Cursor = Cursors.WaitCursor;
      // Eltroljuk a PictureBox komponens aktulis szlessgt s 
      // magassgt.
      kp_szl = pbKp.Width;
      kp_mag = pbKp.Height;
      // Ltrehozunk egy bitkp objektumot, ami pont ekkora mret.
      bmKp = new Bitmap(kp_szl, kp_mag);
      try
      {  // Meghvjuk a szerveren a szmtsokat vgz metdust.
        iSznek = Szerver.MandelSzmt(kp_szl, kp_mag, min_re, 
          min_im, max_re, max_im, max_iter);
        // Belltjuk a bitkp pontjainak sznt a kiszmtott rtkek
        // alapjn.
        for (int y = 0; y < kp_mag; y++)
          for (int x = 0; x < kp_szl; x++)
            bmKp.SetPixel(x, y,
              ColorTranslator.FromWin32(iSznek[y * kp_szl + x]));
        // Az ellltott kpet hozzrendeljk a komponens
        // Image tulajdonsghoz.
        pbKp.Image = bmKp;
      }
      catch (System.Web.Services.Protocols.SoapException se)
      {  // Ha tl nagy a kppontok szneit ler tmb.
        MessageBox.Show("Vltoztassa meg a belltsokat vagy az " +
          "ablak mrett!" +
          "\n\nHiba tpusa: " +se+ "\n\nAz ablak jelenelegi mrete: " +
          this.Width.ToString() + "x" + this.Height.ToString() +
          "\nA kp mrete: " + pbKp.Width.ToString() + "x" +
          pbKp.Height.ToString(),
          "Hiba", MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
      catch (System.Net.WebException)
      {  // Ha nem sikerlt elrni a tvoli gpen a webszolgltatst.
        MessageBox.Show("Nem sikerlt elrni a megadott gpen a" +
          "megadott webszolgltatst!" +
          "\nMdostsa a gp nevt a Mveletek " +
          "men Bellts pontjban!",
          "Hiba", MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
      finally
      { // Belltjuk a norml nyl kurzort, jelezve, hogy a program 
        // befejezte a feladatt.
        Cursor = Cursors.Arrow;
      }
    }

    /// <summary>
    /// rvnytelenn vlt a PictureBox terlete. Fel kell msolni r a 
    /// bitkp objektumban trolt kpet.
    /// </summary>
    private void pbKp_Paint(object sender, PaintEventArgs e)
    { // Az ellltott kpet
      // hozzrendeljk a komponens Image tulajdonsghoz.
      pbKp.Image = bmKp;
    }


  }
}