using System;
using System.Web;
using System.Web.Services;
using System.Web.Services.Protocols;

[WebService(Namespace = "http://JohanykCsaba/")]
[WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
public class MandelbrotSzerver : System.Web.Services.WebService
{
    public MandelbrotSzerver () {

        //Uncomment the following line if using designed components 
        //InitializeComponent(); 
    }

    [WebMethod]
    public string HelloWorld() {
        return "Hello World";
    }
    /// <summary>
    /// Kiszmtja a pont sznt RGB rendszerben.
    /// </summary>
    /// <param name="kpont_re">A pont vals koordintja
    /// (c_re).</param>
    /// <param name="kpont_im">A pont kpzetes 
    /// koordintja (c_im).</param>
    /// <param name="max_iter">Az itercik maximlis 
    /// megengedett szma.</param>
    /// <returns>A pont szne RGB rendszerben.</returns>
    private int SznSzmt(double kpont_re, double kpont_im, int max_iter)
    {	// Hnyadik itercinl tartunk
      int iter_szam = 0;
      // Az iterci kezdetn a komplex szm azonos a kezdponttal
      double pont_re = kpont_re;
      double pont_im = kpont_im;
      // A komplex szm abszolt rtknek a ngyzete
      double pont_abszn;
      do
      {
        iter_szam++;
        pont_re = pont_re * pont_re - pont_im * pont_im + kpont_re;
        pont_im = 2 * pont_re * pont_im + kpont_im;
        pont_abszn = pont_re * pont_re + pont_im * pont_im;
      } while (iter_szam < max_iter && pont_abszn < 4);
      // Az iterci lelltsnak kt felttele van "vagy" kapcsolattal:
      // - a komplex szm abszolt rtke meghaladja a 2-t
      // - az itercik szma elri a megszabott korltot
      // A vgrehajtott itercik szma hatrozza meg a sznt.
      // RGB-ben a legnagyobb szn 0xFFFFFF a legkisebb 0x0
      // Ezt az intervallumot felosztjuk annyi rszre, amennyi a legnagyobb
      // megengedett iterciszm. Ezzel a szn "kvantummal" fogjuk 
      // megszorozni a tnyleges iterciszmot, s a megkapott rtk
      // lesz a megjelentend szn int-ben RGB rendszerben megadva.
      int szin_kvant = 0xFFFFFF / max_iter;
      return szin_kvant * iter_szam;
    }
    /// <summary>
    /// Mandelbrot halmaz szmtst vgez. A szmtst kp_szl*kp_mag 
    /// szm 
    /// pontban vgzi el a [min_re,max_re] s [min_im,max_im] intervallumok 
    /// ltal 
    /// meghatrozott tglalap belsejben. 
    /// </summary>
    /// <param name="kp_szl">Hny pontos lesz a kpernyn vzszintesen 
    ///		a kp.</param>
    /// <param name="kp_mag">Hny pontos lesz a kpernyn fgglegesen 
    ///		a kp.</param>
    /// <param name="min_re">Tglalap bal als sarknak vzszintes 
    ///		koordintja.</param>
    /// <param name="min_im">Tglalap bal als sarknak fggleges 
    ///		koordintja.</param>
    /// <param name="max_re">Tglalap jobb fels sarknak vzszintes 
    ///		koordintja.</param>
    /// <param name="max_im">Tglalap jobb fels sarknak fggleges 
    ///		koordintja.</param>
    /// <param name="Szinek">Kimen paramter. Egydimenzis tmb, ami a 
    ///		kppontok sznt tartalmazza RGB rendszerben.</param>
    /// <param name="max_iter">Megadja, hogy legfeljebb hny iterci
    ///           hajthat 
    ///		vgre egy pont esetn</param>
    [WebMethod]
    public void MandelSzmt(int kp_szl, int kp_mag,
      double min_re, double min_im, double max_re, double max_im,
      out int[] Szinek, int max_iter)
    {	// Ltrehozunk egy tmbt a kppontsznek trolsra.
      Szinek = new int[kp_szl * kp_mag];
      // Meghatrozunk fggleges s vzszintes irnyban egy-egy arny 
      // rtket, amelyek lehetv teszik, hogy minden kpponthoz 
      // kiszmtsunk egy pontot a komplex skon bell kivlasztott 
      //tglalapban.
      double arany_re = (max_re - min_re) / (kp_szl - 1);
      double arany_im = (max_im - min_im) / (kp_mag - 1);
      // Vgighaladunk minden kpponton.
      for (int y = 0; y < kp_mag; y++)
        for (int x = 0; x < kp_szl; x++)
        { // Meghatrozzuk, hogy az adott kpponthoz milyen pont 
          // tartozik a komplex skban.
          double pont_re = min_re + ((double)x * arany_re);
          double pont_im = min_im + ((double)y * arany_im);
          // Meghatrozzuk a kppont sznt RGB rendszerben, s troljuk 
          // egy egydimenzis tmbben.
          Szinek[y * kp_szl + x] = SznSzmt(pont_re, pont_im,
            max_iter);
        }
    }

}
